/**
 * \file dico.c
 * \brief Stupid wrappers for time measurement.
 * \date 2017
 * \author 
 * \copyright WTFPLv2
 */

/* Copyright © 2017 Harenome Razanajato <hrazanajato@unistra.fr>
 *
 * This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar.
 *
 * See http://www.wtfpl.net/ or read below for more details.
 */
/* DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *                    Version 2, December 2004
 *
 * Copyright (C) 2004 Sam Hocevar <sam@hocevar.net>
 *
 * Everyone is permitted to copy and distribute verbatim or modified
 * copies of this license document, and changing it is allowed as long
 * as the name is changed.
 *
 *            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *
 *  0. You just DO WHAT THE FUCK YOU WANT TO.
 */


#include "dico.h"

Dico newDico()
{
	Dico t = (Dico)malloc(sizeof(struct s_dico));
	int i;
	for (i = 0; i < N; i++)
	{
		t->mots[i] = NULL;
		t->mots_collid[i] = nouvlg();
	}
	return t;
}

void freeDico(Dico t)
{
	int i;
	for (i = 0; i < N; i++)
	{
		destroylg(t->mots_collid[i], &free);
	}
	free(t);
}

// compute hash code
unsigned int hash(char *s)
{
	unsigned long h = 0;
	unsigned long r = 1;
	while (*s != 0) { h += r*(*s - 'a' + 1); r++; s++;  }
	return (((22695477 * h + 1)  >> 16) % N);
}


// returns the pointer to the string that has strid as identifier
char  *getStr(Dico t, unsigned int strid)
{
	unsigned int i;
	for (i = 0; i < N; i++)
	{
		if (t->mots[i] != NULL)
		{
			if (identDuMot(t->mots[i]) == strid) return chaineDuMot(t->mots[i]);
			else
			{
				// search in the liste
				for (int j = 0; j < longueur(t->mots_collid[i]); j++)
				{
					listeg l = kiemelg(t->mots_collid[i], j);
					if (identDuMot(elemlg(l)) == strid) return chaineDuMot(elemlg(l));
				}
			}
		}
	}
	return NULL;
}

// returns the identifier of the string str, or OMEGA if not in database
unsigned int getIdent(Dico t, char *str)
{
	unsigned int h = hash(str);
	if (t->mots[h] == NULL) return OMEGA;
	if (strcmp(chaineDuMot(t->mots[h]), str) == 0) return identDuMot(t->mots[h]);
	int r = ranglg(t->mots_collid[h], str, &compareMot);
	if (r == -1) return OMEGA;
	listeg l = kiemelg(t->mots_collid[h], r);
	return identDuMot(elemlg(l));
}

// adds a string str to the database, returns its unique identifier
void adj(Dico t, char *str, unsigned int id)
{
	unsigned int h = hash(str);
	mot *mn = motNouv(str, id);
	if (t->mots[h] == NULL)
	{
		t->mots[h] = mn;
		return;
	}
	t->mots_collid[h] = adjtetelg(t->mots_collid[h], mn, &cloneMot);
	detruireMot(mn);
}

void supStr(Dico t, char *str)
{
	unsigned int h = hash(str);
	if (t->mots[h] == NULL) abort(); // should not happen
	if (strcmp(chaineDuMot(t->mots[h]), str) == 0)
	{
		if (videlg(t->mots_collid[h]))
		{
			detruireMot(t->mots[h]);
			t->mots[h] = NULL;
		}
		else
		{
			detruireMot(t->mots[h]);
			t->mots[h] = cloneMot(elemlg(t->mots_collid[h]));
			t->mots_collid[h] = supkiemelg(t->mots_collid[h], 0,&detruireMot);
		}
		return;
	}
	int r = ranglg(t->mots_collid[h], str, &compareMot);
	if (r == -1) abort(); // should not happen
	t->mots_collid[h] = supkiemelg(t->mots_collid[h], (unsigned int)r, &detruireMot);
}


  
