/**
 * @file relations.c
 * @brief 
 * Groupe TD1 - TP2
 * Nilaina Razafindrambola
 * 22 10 60 17
 * 
 * @paragraph
 * relations.c: definit le point d'entree pour l'application.
 */

typedef int bool;
#define false 0
#define true -1

#include "stdlib.h"
#include "memory.h"
#include "stdio.h"
#include "string.h"

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Exercice 1: Classement des Relations
//

/**
 * @brief 
 * 2 <= parente <= 6
 * < connaisance <= 11
 * < endroit <= 16
*/
typedef enum {
	FRERE = 2, COUSIN, PARENT, ONCLE, EPOUX, AMI, VIT, CONNAIT,
	CHEF, COLLEGUE, LOCATAIRE, TRAVAILLE, PROPRIETAIRE, SITUE, DECOUVERT
} rtype;

/**
 * @brief Table statique par adressage associatif
 * pour le changement en chaine de caractere de `rtype`
*/
char *RTYPE_STRINGS[] = {
		[0] = "NULL",
		[1] = "relation imconnue",
		[FRERE] = "frere ou soeur de",
		[COUSIN] = "cousin de",
		[PARENT] = "parent de",
		[ONCLE] = "oncle de",
		[EPOUX] = "epoux ou epouse de",
		[AMI] = "ami de",
		[VIT] = "vit avec",
		[CONNAIT] = "connait",
		[CHEF] = "chef de",
		[COLLEGUE] = "collegue de",
		[LOCATAIRE] = "locataire",
		[TRAVAILLE] = "travaille a",
		[PROPRIETAIRE] = "proprietaire",
		[SITUE] = "situe a",
		[DECOUVERT] = "decouvert a"
	};

/**
 * @brief Verifie si un rtype est une relation parentale
 * @param id la relation a tester
 * @retval true (est un parent) - false (ne lest pas)
 */
bool est_lien_parente(rtype id) { 
	return ((id >= 2)&&(id <= 6)) ? true : false; 
}

/**
 * @brief Verifie si un rtype est une relation professionelle
 * @param id la relation a tester
 * @retval true (relation professionel) - false (ne l'est pas)
 */
bool est_lien_professionel(rtype id) { 
	return ((id >= 10)&&(id <= 11)) ? true : false;
}

/**
 * @brief Verifie si un rtype est une connaissance
 * @param id la relation a tester
 * @retval true (est une connaissance) - false (ne l'est pas)
 */
bool est_lien_connaissance(rtype id) { 
	return ((id >= 7)&&(id <= 9)) ? true : false;
}

/**
 * @brief Donne la version chaine de caractere de la relation
 * @param id la relation
 * @retval si la relation existe, revoie la chaine de 
 * caractere associee
 * @retval sinon renvoie `??`
 */
char* toStringRelation(rtype id) { 
	if (id < 0 || id > DECOUVERT) {
		return "??";
	}
	return RTYPE_STRINGS[id];
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Exercice 2: Liste de pointeurs
//

/**
 * @brief Structure pour la liste 
 * generique simplement chainee 
 * `e : struct s_node`
 * `*e : listeg`
 */
typedef struct s_node {
	void *val;  // pointeur vers objet quelconque
	struct s_node *suiv;
} *listeg;

/**
 * @brief Cree une nouvelle liste generique
 * @return donne une nouvelle liste generique vide
 */
listeg listegnouv(){ return NULL; }

/**
 * @brief Teste si la liste generique est vide ou non
 */
bool estvide(listeg lst){
	return lst == NULL ? true : false;
}

/**
 * @brief Donne la tete d'une liste generique
 */
void *tete(listeg lst){	
	if(estvide(lst))
		return NULL;
	return lst->val;
}

/**
 * @brief Donne la longueur d'une liste generique
 */
int longueur(listeg lst){
	int count = 0;
	if(!estvide(lst)) // Liste non vide
		while(lst->suiv != NULL){ // Liste avec au moins un element :: count {0,N}
			lst = lst->suiv;
			count++;
		}
	else // Liste vide
		count = -1;
	return count;
}

/**
 * @brief Detruit une liste generique
 */
void detruire(listeg lst){	
	if(!estvide(lst)){ // Liste non vide
		while(lst->suiv != NULL){
			listeg tmp = lst->suiv;
			free(lst);
			lst = tmp;
		}
		free(lst);
	}
}

/**
 * @brief Ajoute un element en tete d'une liste generique
 * @param lst liste generique
 * @param x element a ajouter
 * @return liste avec un element en plus au debut
 */
listeg adjtete(listeg lst, void *x){	
	// Nouveau noeud
	listeg nouv_noeud = (listeg)malloc(sizeof(struct s_node));
	nouv_noeud->val = x;
	nouv_noeud->suiv = NULL;

	// Insertion du nouveau neoud
	nouv_noeud->suiv = lst; // nouveau pointe vers l'ancient
	lst = nouv_noeud; // l'ancient devient le nouveau

	return lst;
}

/**
 * @brief Ajoute un element en queue d'une liste generique
 * @param lst liste generique
 * @param x element a ajouter
 * @return liste avec un element en plus a la fin
 */
listeg adjqueue(listeg lst, void *x){
	// Nouveau noeud
	listeg nouv_noeud = (listeg)malloc(sizeof(struct s_node));
	nouv_noeud->val = x;
	nouv_noeud->suiv = NULL;

	// Insertion du nouveau noeud
	if(estvide(lst)) // Liste vide
		lst = nouv_noeud;
	else { // Liste non vide
		listeg lst_buf = lst;
		while(lst_buf->suiv != NULL)
			lst_buf = lst_buf->suiv;

		lst_buf->suiv = nouv_noeud; // pointeur NULL devient le nouveau
	}

	return lst;
}

/**
 * @brief Supprime la tete de la liste
 * @param lst liste generique
 * @return liste avec un element en moins
 */
listeg suptete(listeg lst){	
	printf("call\n");
	if(estvide(lst))
		return NULL;
	else if(lst->suiv == NULL) {
		free(lst);
		printf("HERE\n");
		return NULL;
	}
	else {
		listeg tete = lst;
		lst = lst->suiv;
		free(tete);
		return lst;
	}
	return NULL;
}

/**
 * @brief Donne l'element tel que
 * l'element de la liste soit `comp` avec `x`
 * @param lst liste genique
 * @param x element a chercher
 * @param comp fonction de comparaison des pour
 * pour le type de la liste `lst`
 * @return si pas trouver donne une nouvelle
 *  liste generique vide
 */
listeg rech(listeg lst, void *x, int(*comp)(void *, void *)){
	if(!estvide(lst)) // Liste non vide
		while(lst != NULL){
			if(comp(lst->val, x) == 0)
				return lst;
			lst = lst->suiv;
		}
	return NULL; // Listw vide
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Exercice 3: Construction du graphe
//

#define LONG_NOM_MAX 64
typedef enum { PERSONNE=1, OBJET, ADRESSE, VILLE } etype;

/**
 * @brief Structure pour un entite
 * `e : struct s_entite`
 * `*e : Entier`
 */
typedef struct s_entite {
	char nom[LONG_NOM_MAX]; /**< le nom de l'entite p.ex " Peugeot 106 "*/
	etype ident; /**< l'identifiant associe, p.ex OBJET */
} *Entite;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// 3.1 les structures de donnees
//

/**
 * @brief Structure pour un sommet
 * `s : struct s_sommet`
 * `*s : Sommet`
 */
typedef struct s_sommet {
	listeg larcs; /**< liste des relations du sommet*/
	void* x; /**< element associe au sommet*/
} *Sommet;

/**
 * @brief Structure pour un arc
 * `a : struct s_arc`
 * `*a : Arc`
 */
typedef struct s_arc {
	rtype t; /**< type de la relation avec `x`*/
	Entite x; /**< Entite associe a la relation */
} *Arc;

/**
 * @brief Structure pour une relation
 * `r : struct s_relations`
 * `*a : Relations`
 */
typedef struct s_relations {
	listeg liste; /**< liste des sommets avec son entite*/
} *Relations;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// 3.2 les constructeurs
//

/**
 * @brief Creation d'entite
 * de nom `s` et de type `e`
 * @return nouvel entite
 */
Entite creerEntite(char *s, etype e){
	Entite ent = (Entite)malloc(sizeof(struct s_entite));	// to free
	strcpy(ent->nom, s);
	ent->ident = e;
	return ent;
}

/**
 * @brief Detruit un entite
 */
void detruireEntite(Entite e){
	if(e != NULL)
		free(e);
}

/**
 * @brief Cree un nouveau sommet avec un entite `e`
 * @return nouveau sommet
 */
Sommet nouvSommet(Entite e){	
	Sommet som = (Sommet)malloc(sizeof(struct s_sommet)); // to free
	som->larcs = NULL;
	som->x = e;
	return som;
}

/**
 * @brief Detruit un sommet
 */
void detruireSommet(Sommet e){
	if(!estvide(e->larcs)){
		detruire(e->larcs);
	}
	// detruireEntite((Entite)e->x);
	free(e);
}

/**
 * @brief Cree un nouvel arc avec un entite et 
 * le type de relation qu'il a avec le sommet
 * @param e entite a creer
 * @param type type de sa relation au sommet
 * @return nouvel arc
 */
Arc nouvArc(Entite e, rtype type){
	Arc arc = (Arc)malloc(sizeof(struct s_sommet)); // to free
	arc->t = type;
	arc->x = e;
	return arc;
}

/**
 * @brief Detruit un arc
 */
void detruireArc(Arc e){
	// if(e != NULL)
	// 	detruireEntite((Entite)e->x);
	free(e);
}

/**
 * @brief Cree une relation et la met dans `g`
 * @param g graphe de la relation
 */
void relationInit(Relations *g){
	(*g) = (Relations)malloc(sizeof(struct s_relations));
	(*g)->liste = listegnouv();
}

/**
 * @brief Detruit une relation - detruit chaque element de la liste de 
 * la relation : destruction des sommets et des arcs
 * a l'interieur
 */
void relationFree(Relations *g){
	listeg lst = (*g)->liste;
	if(!estvide(lst)) // Liste non vide
		while(lst != NULL){
			// liberation de chaque sommet
			if(lst->val != NULL){
				// liberation des arcs dans les sommets
				listeg lstRelationDansSommet = ((Sommet)lst->val)->larcs;
				if(!estvide(lstRelationDansSommet)){
					while(lstRelationDansSommet != NULL){
						// destruction de chaque arc
						detruireArc((Arc)lstRelationDansSommet->val);
						lstRelationDansSommet = lstRelationDansSommet->suiv;
					}
				}
				detruireEntite( (Entite)(((Sommet)lst->val)->x) );
				detruireSommet((Sommet)lst->val);
			}
			lst = lst->suiv;
		}
	detruire((*g)->liste);
	free(*g);
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//3.3 les comparaisons
//

/**
 * @brief compare si un entite est un entite 
 * de nom `string`
 * @param e entite a comparer
 * @param string nom (char*) a comparer a l'entite
 * @retval 0 (sont egales)
*/
int compEntite(void *e, void *string){
	char*nom = ((Entite)e)->nom;
	return strcmp(nom, string);
}

/**
 * @brief compare si un entite du sommet est 
 * un entite de nom `string`
 * @param s sommet a comparer
 * @param string nom (char*) a comparer au sommet
 * @retval 0 (sont egales)
*/
int compSommet(void *s, void *string){
	char*nom = ((Entite)(((Sommet)s)->x))->nom;
	return strcmp(nom, (char*)string);
}

/**
 * @brief compare si un entite de l'arc est 
 * un entite de nom `string`
 * @param a arc a comparer
 * @param string nom (char*) a comparer a l'arc
 * @retval 0 (sont egales)
*/
int compArc(void *a, void *string){
	char*nom = ((Entite)(((Arc)a)->x))->nom;
	return strcmp(nom, string);
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//3.4 ajout d'entites et de relations
//

/**
 * @brief ajoute un entite dans le graphe des relations
 * @param g graphe de la relation
 * @param nom nom de l'entie a ajouter dans la relation
 * @param t type de l'entite a inseret
*/
void adjEntite(Relations g, char *nom, etype t){
	// Verifie si l'entite existe deja ou non
	listeg lstRelation = g->liste;
	if(!estvide(lstRelation)){
		while (lstRelation != NULL)
		{
			int trouve = compSommet((Sommet)lstRelation->val, nom);
			if(trouve == 0){
				// printf("Entite `%s` deja existant dans le graphe\n", nom);
				return;
			}
			lstRelation = lstRelation->suiv;
		}
	}
	// Ajout
	Entite ent = creerEntite(nom, t);
	Sommet som = nouvSommet(ent);
	g->liste = adjqueue(g->liste, (void*)som);
	// printf("Entite `%s` a bien ete inseret \n", nom);
}


/**
 * @brief ajoute une relation a deux entite du graphe
 * - si l'entite n'existe pas, il n'y a pas d'insertion
 * @param g graphe de la relation
 * @param nom1 nom de l'entite du sommet ou on veut mettre
 * la relation
 * @param nom2 nom de l'entie a ajouter dans la relation
 * @param id type de la relation avec `nom1`
 * 
 * @pre id doit etre coherent avec les types des sommets correspondants a x et y
 * p.ex si x est de type OBJET, id ne peut pas etre une relation de parente
 * @pre strcmp(nom1,nom2)!=0
*/
void adjRelation(Relations g, char *nom1, char *nom2, rtype id){
	// Verifie si l'entite existe deja ou non
	listeg lstPersonnes = g->liste;
	if(!estvide(lstPersonnes)){
		while (lstPersonnes != NULL){
			int trouve = compSommet((Sommet)lstPersonnes->val, nom1);
			///////
			// On a trouve l'entite ou on va ajoute la relation
			if(trouve == 0){
				// ajout de l'element a la liste des relation de nom1
				// ent existe deja dans la liste : on le cherche
				Entite ent = NULL;
				listeg lstNom2 = g->liste;
				if(!estvide(lstNom2)){
					while (lstNom2 != NULL){
						trouve = compSommet((Sommet)(lstNom2->val), nom2);
						if(trouve == 0){
							// printf("entite trouve : %s\n", ((Entite)(((Sommet)(lstNom2->val))->x))->nom);
							ent = (Entite)(((Sommet)(lstNom2->val))->x);
						}
						lstNom2 = lstNom2->suiv;
					}
				}

				listeg lstRelations = ((Sommet)lstPersonnes->val)->larcs;
				if(!estvide(lstRelations)){
					while (lstRelations != NULL)
					{
						// Verifie si l'entite existe deja ou non
						trouve = compArc((Arc)lstRelations->val, nom2);
						if(trouve == 0){
							((Arc)(lstRelations->val))->t = id; // mise a jour
							return;
						}
						/////// 
						lstRelations = lstRelations->suiv;
					}
				}
				// ajoute de l'arc
				if(ent != NULL){
					Arc a = nouvArc(ent, id);
					((Sommet)lstPersonnes->val)->larcs = adjqueue(((Sommet)lstPersonnes->val)->larcs, a);
					adjRelation(g, nom2, nom1, id);
					// printf("Insertion de Entite `%s` comme relation de `%s` reussi\n", nom2, nom1);
				} else {
					// printf("Insertion echoue : Entite `%s` n'existe pas encore dans le graphe\n", nom2);
				}
			} // sinon on ne fait rien
			lstPersonnes = lstPersonnes->suiv;
		}

		
	} // sinon on ne fait rien

}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Exercice 4: Explorer les relations entre personnes
//

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// 4.1 listes de relations
//

/**
 * @brief Donne la liste des relations
 * de sommet correspondant a `x`
 * @param g graphe de la relation
 * @param x nom du sommet
 * @return liste de toutes les relations
*/
listeg en_relation(Relations g, char *x)
{
	listeg lst = rech(g->liste, x, compSommet);
	return ((Sommet)(lst->val))->larcs;
}

/**
 * @brief Donne tous les chemins de x a y de 
 * longueur 2 dans le graphe des relations
 * @param g le graphe des relations
 * @param x premier entite de depart
 * @param y dexieme entite d'arrive
 * @return tous les entites de passage entre x et y
*/
listeg chemin2(Relations g, char *x, char *y)
{
	listeg rel = listegnouv();
	listeg xRel = en_relation(g, x);
	if(xRel != NULL){
		while (xRel != NULL){
			char* nomxRel = ((Arc)(xRel->val))->x->nom;
			listeg xRelsecondDeg = en_relation(g, nomxRel);
			while (xRelsecondDeg != NULL){
				char* nomxRelsecondDeg = ((Arc)(xRelsecondDeg->val))->x->nom;
				bool diffx = strcmp(x, nomxRelsecondDeg) != 0 ? true : false; // pas de boucle de relation sur x y p.ex x->z->x
				bool diffy = strcmp(y, nomxRel) != 0 ? true : false; // pas de lien direct
				bool trouve = strcmp(nomxRelsecondDeg, y) == 0 ? true : false; // on a trouver y dans secondDegRelation
				if((trouve)&&(diffx)&&(diffy)){
					rel = adjqueue(rel, ((Arc)(xRel->val))->x);
				}
			xRelsecondDeg = xRelsecondDeg->suiv;
			}

		xRel = xRel->suiv;
		}
	}
	// // lien par un tiers
	// listeg lst = g->liste;
	// while (lst != NULL){
	// 	Entite ent = ((Entite)(((Sommet)lst->val)->x));

	// 	listeg lstarcs = (((Sommet)lst->val)->larcs);
	// 	bool eqx = false;
	// 	bool eqy = false;
	// 	while(lstarcs != NULL) {
	// 		eqx = eqx||strcmp(x, ((Arc)(lstarcs->val))->x->nom) == 0 ? true : false;
	// 		eqy = eqy||strcmp(y, ((Arc)(lstarcs->val))->x->nom) == 0 ? true : false;
	// 	lstarcs = lstarcs->suiv;	
	// 	}
	// 	if(eqx&&eqy)
	// 		rel = adjqueue(rel, ent);
	// lst = lst->suiv;
	// }
	return rel;
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// 4.2 verifier un lien de parente
//

/**
 * @brief Verifie si deux entite ont un lien de
 * parente
 * @pre strcmp(x,y)!=0
*/
bool ont_lien_parente(Relations g, char *x, char *y)
{
	listeg rel = en_relation(g, x);
	while (rel != NULL){
		char* nomRel = ((Arc)(rel->val))->x->nom;
		rtype typeRel = ((Arc)(rel->val))->t;
		bool meme = strcmp(nomRel, y) == 0 ? true : false;
		if((meme)&&(typeRel <= EPOUX)){
			return true;}
	rel = rel->suiv;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// 4.3 tester connaissances
//

/**
 * @brief Verifie si deux personne se connaisent
 * @pre les sommets correspondants a x et y sont de type PERSONNE
 * @pre strcmp(x,y)!=0
*/
bool se_connaissent(Relations g, char *x, char *y)
{
	// lien direct dans les relations
	listeg rel = en_relation(g, x); // pas de liberation : juste une reference
	while (rel != NULL){
		char* nomRel = ((Arc)(rel->val))->x->nom;
		rtype typeRel = ((Arc)(rel->val))->t;
		bool meme = strcmp(nomRel, y) == 0 ? true : false;

		bool appartient = false;
		listeg ch = chemin2(g, x, y);
		listeg lst = ch;
		while(lst != NULL){ // si dans la relation sesond deg
			appartient = appartient || strcmp( ((Entite)(lst->val))->nom, nomRel) == 0 ? true : false;
		lst= lst->suiv;
		}
		detruire(ch);

		if( ((meme)||(appartient)) && (typeRel <= COLLEGUE) ){
			if(appartient)
			return true;
		}
	rel = rel->suiv;
	}
	return false;
}

/**
 * @brief Verifie si deux personne se connaisent probablement
 * @pre les sommets correspondants a x et y sont de type PERSONNE
 * @pre strcmp(x,y)!=0
*/
bool se_connaissent_proba(Relations g, char *x, char *y)
{
	// lien direct dans les relations
	listeg rel = en_relation(g, x);
	while (rel != NULL){
		char* nomRel = ((Arc)(rel->val))->x->nom;
		rtype typeRel = ((Arc)(rel->val))->t;
		bool meme = strcmp(nomRel, y) == 0 ? true : false;

		bool appartient = false;
		listeg ch = chemin2(g, x, y);
			listeg lst = ch;
			while(lst != NULL){ // si dans la relation sesond deg
				appartient = appartient || strcmp( ((Entite)(lst->val))->nom, nomRel) == 0 ? true : false;
			lst= lst->suiv;
			}
		detruire(ch);

		bool parent = false;
		if(appartient){
			bool parentX = ont_lien_parente(g, x, nomRel);
			bool parentY = ont_lien_parente(g, y, nomRel);
			parent = (parentX&&!parentY)||(!parentX&&parentY);
		}
		if( ((!meme)&&(appartient)) && (typeRel <= COLLEGUE) && ( ((Arc)(rel->val))->x->ident == PERSONNE ) && (parent) ){
			if(appartient)
			return true;
		}
	rel = rel->suiv;
	}
	return false;
}

/**
 * @brief Verifie si deux personne se connaisent peut etre
 * @pre les sommets correspondants a x et y sont de type PERSONNE
 * @pre strcmp(x,y)!=0
*/
bool se_connaissent_peutetre(Relations g, char *x, char *y)
{
	listeg ch = chemin2(g, x, y);
		bool proba = se_connaissent_proba(g, x, y);
		bool vide = estvide(ch);
	detruire(ch);
	return (!vide&&proba);
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Exercice 5: Affichages
//

/**
 * @brief affiche une liste generique
*/
void affichelg(listeg l, void(*aff)(void *))
{
	while(l != NULL){
		aff(l->val);
	l = l->suiv;
	}
}

/**
 * @brief affiche un Entite
*/
void afficheEntite(void *x)
{
	// PERSONNE=1, OBJET, ADRESSE, VILLE
	char* type = NULL;
	etype t = ((Entite)x)->ident;
	switch (t)
	{
	case PERSONNE:
		type = "personne";
		break;

	case OBJET:
		type = "objet";
		break;
	
	case ADRESSE:
		type = "adresse";
		break;
	
	case VILLE:
		type = "ville";
		break;

	default:
		type = "NULL";
		break;
	}
	printf("%s:%s\n", ((Entite)x)->nom, type );
}

/**
 * @brief affiche un Arc
*/
void afficheArc(void *x)
{
	printf("--%s-->", toStringRelation(((Arc)x)->t) );
	afficheEntite(((Arc)x)->x);
	printf("\n");
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Exercice 6: Parcours
//
/**
 * @brief affiche les relations du 1er, 
 * 2em et 3em degre d'un entite 
 * @param r graphe de la relation
 * @param x nom de l'entite
*/
void affiche_degre_relations(Relations r, char *x)
{
	printf("%s\n-- 1\n", x);
	listeg rel = en_relation(r, x);
	while (rel != NULL){
		afficheArc(rel->val);
	rel = rel->suiv;
	}

	printf("-- 2\n");
	rel = r->liste;
	while (rel != NULL){
		listeg lst = chemin2(r, x, ((Entite)(((Sommet)rel->val)->x))->nom );
		if(!estvide(lst))
			afficheEntite((Entite)(((Sommet)rel->val)->x));
		detruire(lst);
	rel = rel->suiv;
	}

	printf("-- 3\n");
	rel = r->liste;
	listeg rel1 = r->liste;
	while (rel != NULL){
		while(rel1 != NULL){
			listeg rel2Deg = chemin2(r, ((Entite)(((Sommet)rel->val)->x))->nom, ((Entite)(((Sommet)rel1->val)->x))->nom );
			if(!estvide(rel2Deg)){
				listeg rel3Deg = chemin2(r, x, ((Entite)(((Sommet)rel1->val)->x))->nom );
				if(!estvide(rel3Deg)){
					afficheEntite((Entite)(((Sommet)rel1->val)->x));
				}
				detruire(rel3Deg);
			}
			detruire(rel2Deg);
		rel1 = rel1->suiv;
		}
	rel = rel->suiv;
	}

	printf("\n");
}


/////////////////////////////////////
/////////////////////////////////////
// FONCTION PRINCIPALE
//
int main()
{
	int i,j;
	Relations r; relationInit(&r);
	// ajouter les entites de l'exemple
	char *tabe[] = {"KARL","LUDOVIC","CELINE","CHLOE","GILDAS","CEDRIC","SEVERINE",
		"PEUGEOT 106" ,"1, RUE DE LA RUE","STRASBOURG" };
	for (i = 0; i < 7; i++) adjEntite(r, tabe[i], PERSONNE);
	adjEntite(r, tabe[7], OBJET);
	adjEntite(r, tabe[8], ADRESSE);
	adjEntite(r, tabe[9], VILLE);
	// ajouter les relations de l'exemple
	adjRelation(r, tabe[0], tabe[1], FRERE);
	adjRelation(r, tabe[0], tabe[2], AMI);
	adjRelation(r, tabe[0], tabe[3], CONNAIT);
	adjRelation(r, tabe[0], tabe[5], COUSIN);
	adjRelation(r, tabe[0], tabe[7], PROPRIETAIRE);
	adjRelation(r, tabe[0], tabe[8], PROPRIETAIRE);
	adjRelation(r, tabe[3], tabe[4], VIT);
	adjRelation(r, tabe[5], tabe[6], EPOUX);
	adjRelation(r, tabe[5], tabe[8], LOCATAIRE);
	adjRelation(r, tabe[7], tabe[8], DECOUVERT);
	adjRelation(r, tabe[8], tabe[9], SITUE);

	// explorer les relations
	printf("%s est en relation avec:\n", tabe[0]);
	affichelg(en_relation(r, tabe[0]),afficheArc);
	printf("\n");

	for (i = 0; i < 7; i++) for (j = i + 1; j < 10; j++)
	{
		printf("<%s> et <%s> ont les relations communes:\n", tabe[i], tabe[j]);
		listeg ch = chemin2(r, tabe[i], tabe[j]);
		affichelg(ch, afficheEntite);
		printf("\n");
		detruire(ch);
	}
	printf("\n\n");

	for (i = 0; i < 10; i++) for (j = i + 1; j < 10; j++)
	{
		printf("<%s> et <%s> ont lien de parente: %s\n",
			tabe[i], tabe[j], ont_lien_parente(r, tabe[i], tabe[j]) ? "vrai" : "faux");
	}
	printf("\n");
	for (i = 0; i < 7; i++)
	{
		for (j = i + 1; j < 7; j++)
		{
			printf("<%s> et <%s> se connaissent: %s\n",
				tabe[i], tabe[j], se_connaissent(r, tabe[i], tabe[j]) ? "vrai" : "faux");
			printf("<%s> et <%s> se connaissent tres probablement: %s\n",
				tabe[i], tabe[j], se_connaissent_proba(r, tabe[i], tabe[j]) ? "vrai" : "faux");
			printf("<%s> et <%s> se connaissent peut etre: %s\n",
				tabe[i], tabe[j], se_connaissent_peutetre(r, tabe[i], tabe[j]) ? "vrai" : "faux");
		}
		printf("\n");
	}

	affiche_degre_relations(r, tabe[3]);

	relationFree(&r);

	printf("\nPRESS RETURN\n");
	char buff[64]; fscanf(stdin, "%s", buff);
    return 0;
}
/////////////////////////////////////
/////////////////////////////////////
