// TP3  

/******************************************/
/*
// grapheurbain.c
*/

#include "grapheurbain.h"

// operations internes
grapheurbain _rechCarrefour(grapheurbain g, Nat icar)
{
	if (g == NULL) return NULL;
	if (g->c.icf == icar) return g;
	return _rechCarrefour(g->suiv, icar);
}
ListeSom _rechArc(ListeSom ls, Nat icar)
{
	if (ls == NULL) return NULL;
	if (ls->icf == icar) return ls;
	return _rechArc(ls->suc, icar);
}
ListeSom _rechArcVal(ListeSom ls, Nat irue)
{
	if (ls == NULL) return NULL;
	if (ls->irue == irue) return ls;
	return _rechArcVal(ls->suc, irue);
}
ListeSom _supArc(ListeSom ls, Nat icar)
{
	if (ls == NULL) return NULL;
	if (ls->icf == icar) { ListeSom n = ls->suc;  free(ls);  return n; }
	ls->suc = _supArc(ls->suc, icar);
	return ls;
}
ListeSom _adjArc(ListeSom ls, Nat icar, Nat irue)
{
	ListeSom n = (ListeSom)malloc(sizeof(Strarc));
	if (n == NULL) abort();
	n->icf = icar;
	n->irue = irue;
	n->suc = ls;
	return n;
}

// creation et destruction
grapheurbain grapheurbainNouv()
{
	return NULL;
}
void grapheurbainFree(grapheurbain g)
{
	while(g!=NULL)
	{ 
		grapheurbain cur = g;
		ListeSom ls = cur->c.lsucc;
		while (ls != NULL) { ListeSom n = ls->suc; free(ls); ls = n; }
		ls = cur->c.lpred;
		while (ls != NULL) { ListeSom n = ls->suc; free(ls); ls = n; }
		g = g->suiv;
		free(cur);
	}
}

// operations usuelles sur les graphes
void addCarrefour(grapheurbain *g, Nat icar)
{
	if (existeCarrefour(*g, icar)) return; // already exists
	grapheurbain n = (grapheurbain)malloc(sizeof(Strsom));
	if (n == NULL) abort(); // not enough memory
	n->c.icf = icar;
	n->c.lpred = NULL; // isolated node
	n->c.lsucc = NULL;
	n->suiv = *g;
	*g = n;
}
void supCarrefour(grapheurbain *g, Nat icar)
{
	// only possible if there is no connection
	grapheurbain f = *g;
	if (f == NULL) return; // carrefour icar does not exist
	if (f->c.icf != icar) { f = f->suiv;  supCarrefour(&f, icar); (*g)->suiv = f; return; }
	if (f->c.lpred != NULL || f->c.lsucc != NULL) return; // carrefour must be isolated
	*g = f->suiv;
	free(f);
}

void addArc(grapheurbain g, Nat ico, Nat ice, Nat irue)
{
	grapheurbain f = g;
	grapheurbain fco = _rechCarrefour(f,ico);
	grapheurbain fce = _rechCarrefour(f,ice);
	if (fco == NULL || fce == NULL) return; // carrefour does not exist
	if (_rechArc(fco->c.lsucc, ice) == NULL) // add only if arc does not already exists
	{
		fco->c.lsucc = _adjArc(fco->c.lsucc, ice, irue);
	}
	if (_rechArc(fce->c.lpred, ico) == NULL) // add only if arc does not already exists
	{
		fce->c.lpred = _adjArc(fce->c.lpred, ico, irue);
	}
}

void addArcDoubleSens(grapheurbain g, Nat ico, Nat ice, Nat irue)
{
	addArc(g, ico, ice, irue);
	addArc(g, ice, ico, irue);
}

void supArc(grapheurbain g, Nat ico, Nat ice)
{
	grapheurbain f = g;
	grapheurbain fco = _rechCarrefour(f, ico);
	grapheurbain fce = _rechCarrefour(f, ice);
	if (fco == NULL || fce == NULL) return; // carrefour does not exist
	fco->c.lsucc = _supArc(fco->c.lsucc, ice);
	fce->c.lpred = _supArc(fce->c.lpred, ico);
}

boolean existeArc(grapheurbain g, Nat ico, Nat ice)
{
	grapheurbain fco = _rechCarrefour(g, ico);
	grapheurbain fce = _rechCarrefour(g, ice);
	if (fco == NULL || fce == NULL) abort(); // carrefour does not exist
	return _rechArc(fco->c.lsucc, ice) != NULL;
}
boolean existeCarrefour(grapheurbain g, Nat icar)
{
	return _rechCarrefour(g, icar) != NULL;
}

// operations sur les liste de succ et pred
// les carrefours associes a une rue
listeui carrefoursDeRue(grapheurbain g, Nat irue)
{
	listeui res = nouvlui();
	while (g != NULL)
	{
		if (_rechArcVal(g->c.lsucc, irue) != NULL)
		{
			if (ranglui(res, g->c.icf) == -1) res = adjtetelui(res, g->c.icf);
		}
		g = g->suiv;
	}
	return res;
}
// les rues incidentes en un carrefour
listeui ruesDeCarrefour(grapheurbain g, Nat icar)
{
	listeui res = nouvlui();
	grapheurbain f = _rechCarrefour(g, icar);
	ListeSom ls = f->c.lsucc;
	while (ls != NULL)
	{
		if (ranglui(res, ls->irue) == -1) res = adjtetelui(res, ls->irue);
		ls = ls->suc;
	}
	ListeSom lp = f->c.lpred;
	while (lp != NULL)
	{
		if (ranglui(res, lp->irue) == -1) res = adjtetelui(res, lp->irue);
		lp = lp->suc;
	}
	return res;
}

listeui ruesCroisement(grapheurbain g, Nat irue)
{
	listeui res = nouvlui();
	listeui lc = carrefoursDeRue(g, irue);
	for (int i = 0; i < longlui(lc); i++)
	{
		Nat icar = kiemelui(lc, i);
		listeui lr = ruesDeCarrefour(g, icar);
		for (int j = 0; j < longlui(lr); j++)
		{
			Nat jrue = kiemelui(lr, j);
			if (jrue!=irue && ranglui(res, jrue) == -1) res = adjtetelui(res, jrue);
		}
		destroylui(lr);
	}
	destroylui(lc);
	return res;
}


