
#include "stringtab.h"

/******************************************/
/*
// stringtab.c 
Table de chaines de caracteres
*/


StringTab StringTabNouv()
{
	StringTab t = (StringTab)malloc(sizeof(struct s_stringtab));
	if (t==NULL) { printf("memory allocation error.\n"); return NULL; }
	t->n_elem =0;
	t->curr_id=1;
	t->max_elem= N_ELEM_INIT;
	t->tab = (couple *)malloc(sizeof(couple)*t->max_elem);
	if (t->tab==NULL) { printf("memory allocation error.\n"); free(t); return NULL; }
	return t;
}

void StringTabFree(StringTab t)
{
	free(t->tab);
	free(t);
}

Nat StringTabCardinal(StringTab t) { return t->n_elem; }

// usage interne
int StringTabRechPos(StringTab t, String str)
{
	int i;
	for (i=0; i<t->n_elem; i++)
		if (strcmp(t->tab[i].chaine, str) == 0) return i;
	return -1; // not found, return value out of range
}
// usage interne
int StringTabPos(StringTab t, Nat index)
{
	int a = 0, b = t->n_elem-1;
	while (a<b)
	{
		int m = (a+b)/2;
		if (t->tab[m].ident==index) return m;
		if (t->tab[m].ident<index) a=m+1; else b=m-1;
	}
	if (t->tab[a].ident==index) return a;
	else return -1; // not found, return value out of range	
}

// adjonction
Nat StringTabAdd(StringTab t, String str)
{
	if (t->n_elem>=t->max_elem) // debordement memoire
	{
		t->tab=(couple *)realloc(t->tab, sizeof(couple)*(t->max_elem+ N_ELEM_INIT));
		if (t->tab==NULL) { printf("memory allocation error.\n"); exit(1); }
		t->max_elem+= N_ELEM_INIT;
	}
	// adjonction toujours en queue car ident le plus grand
	t->tab[t->n_elem].chaine = (String)malloc(strlen(str) + 1);
	strcpy(t->tab[t->n_elem].chaine,str);
	t->tab[t->n_elem].ident=t->curr_id;
	t->n_elem++;
	t->curr_id++;
	return t->curr_id-1;
}

// acces a la valeur, le resultat est dans str
Nat StringTabRech(StringTab t, String str)
{
	int pos = StringTabRechPos(t, str);
	if (pos == -1) return 0;
	return t->tab[pos].ident;
}
// recherche par clef
String StringTabGetChaine(StringTab t, Nat index)
{
	
	int i= StringTabPos(t,index);
	if (i==-1) return NULL; // not found, return value out of range
	else return t->tab[i].chaine;
}

// supression
void StringTabSup(StringTab t, String str)
{
	int i = StringTabRechPos(t,str);
	if (i==-1) return;
	unsigned int tindex=i;
	free(t->tab[tindex].chaine);
	// supression dans un tableau->dcaler les cases du tableau
	for (i=tindex; i<t->n_elem-1; i++) t->tab[i]=t->tab[i+1];
	t->n_elem--;
}



